@@ -1,57 +1,76 @@
+from __future__ import annotations
+
 import contextvars
 from contextlib import AsyncExitStack
-from typing import AsyncGenerator, Awaitable, Callable, List, Union
+from typing import (
+    Any,
+    AsyncGenerator,
+    AsyncIterator,
+    Generator,
+)
 
 import anyio
 import pytest
+from anyio.abc import TaskStatus
 
 from starlette.applications import Starlette
 from starlette.background import BackgroundTask
-from starlette.middleware import Middleware
+from starlette.middleware import Middleware, _MiddlewareClass
 from starlette.middleware.base import BaseHTTPMiddleware, RequestResponseEndpoint
-from starlette.requests import Request
+from starlette.requests import ClientDisconnect, Request
 from starlette.responses import PlainTextResponse, Response, StreamingResponse
 from starlette.routing import Route, WebSocketRoute
 from starlette.testclient import TestClient
 from starlette.types import ASGIApp, Message, Receive, Scope, Send
+from starlette.websockets import WebSocket
+from tests.types import TestClientFactory
 
 
 class CustomMiddleware(BaseHTTPMiddleware):
-    async def dispatch(self, request, call_next):
+    async def dispatch(
+        self,
+        request: Request,
+        call_next: RequestResponseEndpoint,
+    ) -> Response:
         response = await call_next(request)
         response.headers["Custom-Header"] = "Example"
         return response
 
 
-def homepage(request):
+def homepage(request: Request) -> PlainTextResponse:
     return PlainTextResponse("Homepage")
 
 
-def exc(request):
+def exc(request: Request) -> None:
     raise Exception("Exc")
 
 
-def exc_stream(request):
+def exc_stream(request: Request) -> StreamingResponse:
     return StreamingResponse(_generate_faulty_stream())
 
 
-def _generate_faulty_stream():
+def _generate_faulty_stream() -> Generator[bytes, None, None]:
     yield b"Ok"
     raise Exception("Faulty Stream")
 
 
 class NoResponse:
-    def __init__(self, scope, receive, send):
+    def __init__(
+        self,
+        scope: Scope,
+        receive: Receive,
+        send: Send,
+    ):
         pass
 
-    def __await__(self):
+    def __await__(self) -> Generator[Any, None, None]:
         return self.dispatch().__await__()
 
-    async def dispatch(self):
+    async def dispatch(self) -> None:
         pass
 
 
-async def websocket_endpoint(session):
+async def websocket_endpoint(session: WebSocket) -> None:
     await session.accept()
     await session.send_text("Hello, world!")
     await session.close()
@@ -69,7 +88,7 @@ app = Starlette(
 )
 
 
-def test_custom_middleware(test_client_factory):
+def test_custom_middleware(test_client_factory: TestClientFactory) -> None:
     client = test_client_factory(app)
     response = client.get("/")
     assert response.headers["Custom-Header"] == "Example"
@@ -90,30 +109,44 @@ def test_custom_middleware(test_client_factory):
         assert text == "Hello, world!"
 
 
-def test_state_data_across_multiple_middlewares(test_client_factory):
+def test_state_data_across_multiple_middlewares(
+    test_client_factory: TestClientFactory,
+) -> None:
     expected_value1 = "foo"
     expected_value2 = "bar"
 
     class aMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request, call_next):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             request.state.foo = expected_value1
             response = await call_next(request)
             return response
 
     class bMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request, call_next):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             request.state.bar = expected_value2
             response = await call_next(request)
             response.headers["X-State-Foo"] = request.state.foo
             return response
 
     class cMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request, call_next):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             response = await call_next(request)
             response.headers["X-State-Bar"] = request.state.bar
             return response
 
-    def homepage(request):
+    def homepage(request: Request) -> PlainTextResponse:
         return PlainTextResponse("OK")
 
     app = Starlette(
@@ -132,23 +165,25 @@ def test_state_data_across_multiple_middlewares(test_client_factory):
     assert response.headers["X-State-Bar"] == expected_value2
 
 
-def test_app_middleware_argument(test_client_factory):
-    def homepage(request):
+def test_app_middleware_argument(test_client_factory: TestClientFactory) -> None:
+    def homepage(request: Request) -> PlainTextResponse:
         return PlainTextResponse("Homepage")
 
-    app = Starlette(
-        routes=[Route("/", homepage)], middleware=[Middleware(CustomMiddleware)]
-    )
+    app = Starlette(routes=[Route("/", homepage)], middleware=[Middleware(CustomMiddleware)])
 
     client = test_client_factory(app)
     response = client.get("/")
     assert response.headers["Custom-Header"] == "Example"
 
 
-def test_fully_evaluated_response(test_client_factory):
+def test_fully_evaluated_response(test_client_factory: TestClientFactory) -> None:
     # Test for https://github.com/encode/starlette/issues/1022
     class CustomMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request, call_next):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> PlainTextResponse:
             await call_next(request)
             return PlainTextResponse("Custom")
 
@@ -173,7 +208,11 @@ class CustomMiddlewareWithoutBaseHTTPMiddleware:
 
 
 class CustomMiddlewareUsingBaseHTTPMiddleware(BaseHTTPMiddleware):
-    async def dispatch(self, request, call_next):
+    async def dispatch(
+        self,
+        request: Request,
+        call_next: RequestResponseEndpoint,
+    ) -> Response:
         ctxvar.set("set by middleware")
         resp = await call_next(request)
         assert ctxvar.get() == "set by endpoint"
@@ -196,18 +235,19 @@ class CustomMiddlewareUsingBaseHTTPMiddleware(BaseHTTPMiddleware):
         ),
     ],
 )
-def test_contextvars(test_client_factory, middleware_cls: type):
+def test_contextvars(
+    test_client_factory: TestClientFactory,
+    middleware_cls: type[_MiddlewareClass[Any]],
+) -> None:
     # this has to be an async endpoint because Starlette calls run_in_threadpool
     # on sync endpoints which has it's own set of peculiarities w.r.t propagating
     # contextvars (it propagates them forwards but not backwards)
-    async def homepage(request):
+    async def homepage(request: Request) -> PlainTextResponse:
         assert ctxvar.get() == "set by middleware"
         ctxvar.set("set by endpoint")
         return PlainTextResponse("Homepage")
 
-    app = Starlette(
-        middleware=[Middleware(middleware_cls)], routes=[Route("/", homepage)]
-    )
+    app = Starlette(middleware=[Middleware(middleware_cls)], routes=[Route("/", homepage)])
 
     client = test_client_factory(app)
     response = client.get("/")
@@ -215,74 +255,24 @@ def test_contextvars(test_client_factory, middleware_cls: type):
 
 
 @pytest.mark.anyio
-async def test_run_background_tasks_even_if_client_disconnects():
+async def test_run_background_tasks_even_if_client_disconnects() -> None:
     # test for https://github.com/encode/starlette/issues/1438
-    request_body_sent = False
     response_complete = anyio.Event()
     background_task_run = anyio.Event()
 
-    async def sleep_and_set():
+    async def sleep_and_set() -> None:
         # small delay to give BaseHTTPMiddleware a chance to cancel us
         # this is required to make the test fail prior to fixing the issue
         # so do not be surprised if you remove it and the test still passes
         await anyio.sleep(0.1)
         background_task_run.set()
 
-    async def endpoint_with_background_task(_):
+    async def endpoint_with_background_task(_: Request) -> PlainTextResponse:
         return PlainTextResponse(background=BackgroundTask(sleep_and_set))
 
-    async def passthrough(request, call_next):
-        return await call_next(request)
-
-    app = Starlette(
-        middleware=[Middleware(BaseHTTPMiddleware, dispatch=passthrough)],
-        routes=[Route("/", endpoint_with_background_task)],
-    )
-
-    scope = {
-        "type": "http",
-        "version": "3",
-        "method": "GET",
-        "path": "/",
-    }
-
-    async def receive():
-        nonlocal request_body_sent
-        if not request_body_sent:
-            request_body_sent = True
-            return {"type": "http.request", "body": b"", "more_body": False}
-        # We simulate a client that disconnects immediately after receiving the response
-        await response_complete.wait()
-        return {"type": "http.disconnect"}
-
-    async def send(message):
-        if message["type"] == "http.response.body":
-            if not message.get("more_body", False):
-                response_complete.set()
-
-    await app(scope, receive, send)
-
-    assert background_task_run.is_set()
-
-
-@pytest.mark.anyio
-async def test_do_not_block_on_background_tasks():
-    request_body_sent = False
-    response_complete = anyio.Event()
-    events: List[Union[str, Message]] = []
-
-    async def sleep_and_set():
-        events.append("Background task started")
-        await anyio.sleep(0.1)
-        events.append("Background task finished")
-
-    async def endpoint_with_background_task(_):
-        return PlainTextResponse(
-            content="Hello", background=BackgroundTask(sleep_and_set)
-        )
-
     async def passthrough(
-        request: Request, call_next: Callable[[Request], Awaitable[Response]]
+        request: Request,
+        call_next: RequestResponseEndpoint,
     ) -> Response:
         return await call_next(request)
 
@@ -299,14 +289,50 @@ async def test_do_not_block_on_background_tasks():
     }
 
     async def receive() -> Message:
-        nonlocal request_body_sent
-        if not request_body_sent:
-            request_body_sent = True
-            return {"type": "http.request", "body": b"", "more_body": False}
-        await response_complete.wait()
-        return {"type": "http.disconnect"}
+        raise NotImplementedError("Should not be called!")
 
-    async def send(message: Message):
+    async def send(message: Message) -> None:
+        if message["type"] == "http.response.body":
+            if not message.get("more_body", False):
+                response_complete.set()
+
+    await app(scope, receive, send)
+
+    assert background_task_run.is_set()
+
+
+@pytest.mark.anyio
+async def test_do_not_block_on_background_tasks() -> None:
+    response_complete = anyio.Event()
+    events: list[str | Message] = []
+
+    async def sleep_and_set() -> None:
+        events.append("Background task started")
+        await anyio.sleep(0.1)
+        events.append("Background task finished")
+
+    async def endpoint_with_background_task(_: Request) -> PlainTextResponse:
+        return PlainTextResponse(content="Hello", background=BackgroundTask(sleep_and_set))
+
+    async def passthrough(request: Request, call_next: RequestResponseEndpoint) -> Response:
+        return await call_next(request)
+
+    app = Starlette(
+        middleware=[Middleware(BaseHTTPMiddleware, dispatch=passthrough)],
+        routes=[Route("/", endpoint_with_background_task)],
+    )
+
+    scope = {
+        "type": "http",
+        "version": "3",
+        "method": "GET",
+        "path": "/",
+    }
+
+    async def receive() -> Message:
+        raise NotImplementedError("Should not be called!")
+
+    async def send(message: Message) -> None:
         if message["type"] == "http.response.body":
             events.append(message)
             if not message.get("more_body", False):
@@ -331,13 +357,12 @@ async def test_do_not_block_on_background_tasks():
 
 
 @pytest.mark.anyio
-async def test_run_context_manager_exit_even_if_client_disconnects():
+async def test_run_context_manager_exit_even_if_client_disconnects() -> None:
     # test for https://github.com/encode/starlette/issues/1678#issuecomment-1172916042
-    request_body_sent = False
     response_complete = anyio.Event()
     context_manager_exited = anyio.Event()
 
-    async def sleep_and_set():
+    async def sleep_and_set() -> None:
         # small delay to give BaseHTTPMiddleware a chance to cancel us
         # this is required to make the test fail prior to fixing the issue
         # so do not be surprised if you remove it and the test still passes
@@ -345,18 +370,21 @@ async def test_run_context_manager_exit_even_if_client_disconnects():
         context_manager_exited.set()
 
     class ContextManagerMiddleware:
-        def __init__(self, app):
+        def __init__(self, app: ASGIApp):
             self.app = app
 
-        async def __call__(self, scope: Scope, receive: Receive, send: Send):
+        async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
             async with AsyncExitStack() as stack:
                 stack.push_async_callback(sleep_and_set)
                 await self.app(scope, receive, send)
 
-    async def simple_endpoint(_):
+    async def simple_endpoint(_: Request) -> PlainTextResponse:
         return PlainTextResponse(background=BackgroundTask(sleep_and_set))
 
-    async def passthrough(request, call_next):
+    async def passthrough(
+        request: Request,
+        call_next: RequestResponseEndpoint,
+    ) -> Response:
         return await call_next(request)
 
     app = Starlette(
@@ -374,16 +402,10 @@ async def test_run_context_manager_exit_even_if_client_disconnects():
         "path": "/",
     }
 
-    async def receive():
-        nonlocal request_body_sent
-        if not request_body_sent:
-            request_body_sent = True
-            return {"type": "http.request", "body": b"", "more_body": False}
-        # We simulate a client that disconnects immediately after receiving the response
-        await response_complete.wait()
-        return {"type": "http.disconnect"}
+    async def receive() -> Message:
+        raise NotImplementedError("Should not be called!")
 
-    async def send(message):
+    async def send(message: Message) -> None:
         if message["type"] == "http.response.body":
             if not message.get("more_body", False):
                 response_complete.set()
@@ -393,13 +415,34 @@ async def test_run_context_manager_exit_even_if_client_disconnects():
     assert context_manager_exited.is_set()
 
 
-def test_app_receives_http_disconnect_while_sending_if_discarded(test_client_factory):
+def test_app_receives_http_disconnect_while_sending_if_discarded(
+    test_client_factory: TestClientFactory,
+) -> None:
     class DiscardingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request, call_next):
-            await call_next(request)
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: Any,
+        ) -> PlainTextResponse:
+            # As a matter of ordering, this test targets the case where the downstream
+            # app response is discarded while it is sending a response body.
+            # We need to wait for the downstream app to begin sending a response body
+            # before sending the middleware response that will overwrite the downstream
+            # response.
+            downstream_app_response = await call_next(request)
+            body_generator = downstream_app_response.body_iterator
+            try:
+                await body_generator.__anext__()
+            finally:
+                await body_generator.aclose()
+
             return PlainTextResponse("Custom")
 
-    async def downstream_app(scope, receive, send):
+    async def downstream_app(
+        scope: Scope,
+        receive: Receive,
+        send: Send,
+    ) -> None:
         await send(
             {
                 "type": "http.response.start",
@@ -411,17 +454,24 @@ def test_app_receives_http_disconnect_while_sending_if_discarded(test_client_fac
         )
         async with anyio.create_task_group() as task_group:
 
-            async def cancel_on_disconnect():
+            async def cancel_on_disconnect(
+                *,
+                task_status: TaskStatus[None] = anyio.TASK_STATUS_IGNORED,
+            ) -> None:
+                task_status.started()
                 while True:
                     message = await receive()
                     if message["type"] == "http.disconnect":
                         task_group.cancel_scope.cancel()
                         break
 
-            task_group.start_soon(cancel_on_disconnect)
+            # Using start instead of start_soon to ensure that
+            # cancel_on_disconnect is scheduled by the event loop
+            # before we start returning the body
+            await task_group.start(cancel_on_disconnect)
 
             # A timeout is set for 0.1 second in order to ensure that
-            # cancel_on_disconnect is scheduled by the event loop
+            # we never deadlock the test run in an infinite loop
             with anyio.move_on_after(0.1):
                 while True:
                     await send(
@@ -432,9 +482,7 @@ def test_app_receives_http_disconnect_while_sending_if_discarded(test_client_fac
                         }
                     )
 
-            pytest.fail(
-                "http.disconnect should have been received and canceled the scope"
-            )  # pragma: no cover
+            pytest.fail("http.disconnect should have been received and canceled the scope")  # pragma: no cover
 
     app = DiscardingMiddleware(downstream_app)
 
@@ -443,13 +491,23 @@ def test_app_receives_http_disconnect_while_sending_if_discarded(test_client_fac
     assert response.text == "Custom"
 
 
-def test_app_receives_http_disconnect_after_sending_if_discarded(test_client_factory):
+def test_app_receives_http_disconnect_after_sending_if_discarded(
+    test_client_factory: TestClientFactory,
+) -> None:
     class DiscardingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request, call_next):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> PlainTextResponse:
             await call_next(request)
             return PlainTextResponse("Custom")
 
-    async def downstream_app(scope, receive, send):
+    async def downstream_app(
+        scope: Scope,
+        receive: Receive,
+        send: Send,
+    ) -> None:
         await send(
             {
                 "type": "http.response.start",
@@ -484,9 +542,9 @@ def test_app_receives_http_disconnect_after_sending_if_discarded(test_client_fac
 
 
 def test_read_request_stream_in_app_after_middleware_calls_stream(
-    test_client_factory: Callable[[ASGIApp], TestClient]
+    test_client_factory: TestClientFactory,
 ) -> None:
-    async def homepage(request: Request):
+    async def homepage(request: Request) -> PlainTextResponse:
         expected = [b""]
         async for chunk in request.stream():
             assert chunk == expected.pop(0)
@@ -494,7 +552,11 @@ def test_read_request_stream_in_app_after_middleware_calls_stream(
         return PlainTextResponse("Homepage")
 
     class ConsumingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             expected = [b"a", b""]
             async for chunk in request.stream():
                 assert chunk == expected.pop(0)
@@ -512,9 +574,9 @@ def test_read_request_stream_in_app_after_middleware_calls_stream(
 
 
 def test_read_request_stream_in_app_after_middleware_calls_body(
-    test_client_factory: Callable[[ASGIApp], TestClient]
+    test_client_factory: TestClientFactory,
 ) -> None:
-    async def homepage(request: Request):
+    async def homepage(request: Request) -> PlainTextResponse:
         expected = [b"a", b""]
         async for chunk in request.stream():
             assert chunk == expected.pop(0)
@@ -522,7 +584,11 @@ def test_read_request_stream_in_app_after_middleware_calls_body(
         return PlainTextResponse("Homepage")
 
     class ConsumingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             assert await request.body() == b"a"
             return await call_next(request)
 
@@ -537,14 +603,18 @@ def test_read_request_stream_in_app_after_middleware_calls_body(
 
 
 def test_read_request_body_in_app_after_middleware_calls_stream(
-    test_client_factory: Callable[[ASGIApp], TestClient]
+    test_client_factory: TestClientFactory,
 ) -> None:
-    async def homepage(request: Request):
+    async def homepage(request: Request) -> PlainTextResponse:
         assert await request.body() == b""
         return PlainTextResponse("Homepage")
 
     class ConsumingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             expected = [b"a", b""]
             async for chunk in request.stream():
                 assert chunk == expected.pop(0)
@@ -562,14 +632,18 @@ def test_read_request_body_in_app_after_middleware_calls_stream(
 
 
 def test_read_request_body_in_app_after_middleware_calls_body(
-    test_client_factory: Callable[[ASGIApp], TestClient]
+    test_client_factory: TestClientFactory,
 ) -> None:
-    async def homepage(request: Request):
+    async def homepage(request: Request) -> PlainTextResponse:
         assert await request.body() == b"a"
         return PlainTextResponse("Homepage")
 
     class ConsumingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             assert await request.body() == b"a"
             return await call_next(request)
 
@@ -584,9 +658,9 @@ def test_read_request_body_in_app_after_middleware_calls_body(
 
 
 def test_read_request_stream_in_dispatch_after_app_calls_stream(
-    test_client_factory: Callable[[ASGIApp], TestClient]
+    test_client_factory: TestClientFactory,
 ) -> None:
-    async def homepage(request: Request):
+    async def homepage(request: Request) -> PlainTextResponse:
         expected = [b"a", b""]
         async for chunk in request.stream():
             assert chunk == expected.pop(0)
@@ -594,7 +668,11 @@ def test_read_request_stream_in_dispatch_after_app_calls_stream(
         return PlainTextResponse("Homepage")
 
     class ConsumingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             resp = await call_next(request)
             with pytest.raises(RuntimeError, match="Stream consumed"):
                 async for _ in request.stream():
@@ -612,14 +690,18 @@ def test_read_request_stream_in_dispatch_after_app_calls_stream(
 
 
 def test_read_request_stream_in_dispatch_after_app_calls_body(
-    test_client_factory: Callable[[ASGIApp], TestClient]
+    test_client_factory: TestClientFactory,
 ) -> None:
-    async def homepage(request: Request):
+    async def homepage(request: Request) -> PlainTextResponse:
         assert await request.body() == b"a"
         return PlainTextResponse("Homepage")
 
     class ConsumingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             resp = await call_next(request)
             with pytest.raises(RuntimeError, match="Stream consumed"):
                 async for _ in request.stream():
@@ -646,9 +728,13 @@ async def test_read_request_stream_in_dispatch_wrapping_app_calls_body() -> None
         await Response()(scope, receive, send)
 
     class ConsumingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             expected = b"1"
-            response: Union[Response, None] = None
+            response: Response | None = None
             async for chunk in request.stream():
                 assert chunk == expected
                 if expected == b"1":
@@ -663,9 +749,11 @@ async def test_read_request_stream_in_dispatch_wrapping_app_calls_body() -> None
         yield {"type": "http.request", "body": b"1", "more_body": True}
         yield {"type": "http.request", "body": b"2", "more_body": True}
         yield {"type": "http.request", "body": b"3"}
-        await anyio.sleep(float("inf"))
+        raise AssertionError(  # pragma: no cover
+            "Should not be called, no need to poll for disconnect"
+        )
 
-    sent: List[Message] = []
+    sent: list[Message] = []
 
     async def send(msg: Message) -> None:
         sent.append(msg)
@@ -689,18 +777,20 @@ async def test_read_request_stream_in_dispatch_wrapping_app_calls_body() -> None
     await rcv_stream.aclose()
 
 
-def test_read_request_stream_in_dispatch_after_app_calls_body_with_middleware_calling_body_before_call_next(  # noqa: E501
-    test_client_factory: Callable[[ASGIApp], TestClient]
+def test_read_request_stream_in_dispatch_after_app_calls_body_with_middleware_calling_body_before_call_next(
+    test_client_factory: TestClientFactory,
 ) -> None:
-    async def homepage(request: Request):
+    async def homepage(request: Request) -> PlainTextResponse:
         assert await request.body() == b"a"
         return PlainTextResponse("Homepage")
 
     class ConsumingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
-            assert (
-                await request.body() == b"a"
-            )  # this buffers the request body in memory
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
+            assert await request.body() == b"a"  # this buffers the request body in memory
             resp = await call_next(request)
             async for chunk in request.stream():
                 if chunk:
@@ -717,18 +807,20 @@ def test_read_request_stream_in_dispatch_after_app_calls_body_with_middleware_ca
     assert response.status_code == 200
 
 
-def test_read_request_body_in_dispatch_after_app_calls_body_with_middleware_calling_body_before_call_next(  # noqa: E501
-    test_client_factory: Callable[[ASGIApp], TestClient]
+def test_read_request_body_in_dispatch_after_app_calls_body_with_middleware_calling_body_before_call_next(
+    test_client_factory: TestClientFactory,
 ) -> None:
-    async def homepage(request: Request):
+    async def homepage(request: Request) -> PlainTextResponse:
         assert await request.body() == b"a"
         return PlainTextResponse("Homepage")
 
     class ConsumingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
-            assert (
-                await request.body() == b"a"
-            )  # this buffers the request body in memory
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
+            assert await request.body() == b"a"  # this buffers the request body in memory
             resp = await call_next(request)
             assert await request.body() == b"a"  # no problem here
             return resp
@@ -758,7 +850,11 @@ async def test_read_request_disconnected_client() -> None:
         await Response()(scope, receive, send)
 
     class ConsumingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             response = await call_next(request)
             disconnected = await request.is_disconnected()
             assert disconnected is True
@@ -770,7 +866,7 @@ async def test_read_request_disconnected_client() -> None:
         yield {"type": "http.disconnect"}
         raise AssertionError("Should not be called, would hang")  # pragma: no cover
 
-    async def send(msg: Message):
+    async def send(msg: Message) -> None:
         if msg["type"] == "http.response.start":
             assert msg["status"] == 200
 
@@ -794,7 +890,11 @@ async def test_read_request_disconnected_after_consuming_steam() -> None:
         await Response()(scope, receive, send)
 
     class ConsumingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             await request.body()
             disconnected = await request.is_disconnected()
             assert disconnected is True
@@ -808,7 +908,7 @@ async def test_read_request_disconnected_after_consuming_steam() -> None:
         yield {"type": "http.disconnect"}
         raise AssertionError("Should not be called, would hang")  # pragma: no cover
 
-    async def send(msg: Message):
+    async def send(msg: Message) -> None:
         if msg["type"] == "http.response.start":
             assert msg["status"] == 200
 
@@ -822,7 +922,7 @@ async def test_read_request_disconnected_after_consuming_steam() -> None:
 
 
 def test_downstream_middleware_modifies_receive(
-    test_client_factory: Callable[[ASGIApp], TestClient]
+    test_client_factory: TestClientFactory,
 ) -> None:
     """If a downstream middleware modifies receive() the final ASGI app
     should see the modified version.
@@ -835,7 +935,11 @@ def test_downstream_middleware_modifies_receive(
         await Response()(scope, receive, send)
 
     class ConsumingMiddleware(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             body = await request.body()
             assert body == b"foo "
             return await call_next(request)
@@ -858,17 +962,18 @@ def test_downstream_middleware_modifies_receive(
     assert resp.status_code == 200
 
 
-CallNext = Callable[[Request], Awaitable[Response]]
-
-
 def test_pr_1519_comment_1236166180_example() -> None:
     """
     https://github.com/encode/starlette/pull/1519#issuecomment-1236166180
     """
-    bodies: List[bytes] = []
+    bodies: list[bytes] = []
 
     class LogRequestBodySize(BaseHTTPMiddleware):
-        async def dispatch(self, request: Request, call_next: CallNext) -> Response:
+        async def dispatch(
+            self,
+            request: Request,
+            call_next: RequestResponseEndpoint,
+        ) -> Response:
             print(len(await request.body()))
             return await call_next(request)
 
@@ -897,3 +1002,159 @@ def test_pr_1519_comment_1236166180_example() -> None:
     resp.raise_for_status()
 
     assert bodies == [b"Hello, World!-foo"]
+
+
+@pytest.mark.anyio
+async def test_multiple_middlewares_stacked_client_disconnected() -> None:
+    """
+    Tests for:
+    - https://github.com/encode/starlette/issues/2516
+    - https://github.com/encode/starlette/pull/2687
+    """
+    ordered_events: list[str] = []
+    unordered_events: list[str] = []
+
+    class MyMiddleware(BaseHTTPMiddleware):
+        def __init__(self, app: ASGIApp, version: int) -> None:
+            self.version = version
+            super().__init__(app)
+
+        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint) -> Response:
+            ordered_events.append(f"{self.version}:STARTED")
+            res = await call_next(request)
+            ordered_events.append(f"{self.version}:COMPLETED")
+
+            def background() -> None:
+                unordered_events.append(f"{self.version}:BACKGROUND")
+
+            assert res.background is None
+            res.background = BackgroundTask(background)
+            return res
+
+    async def sleepy(request: Request) -> Response:
+        try:
+            await request.body()
+        except ClientDisconnect:
+            pass
+        else:  # pragma: no cover
+            raise AssertionError("Should have raised ClientDisconnect")
+        return Response(b"")
+
+    app = Starlette(
+        routes=[Route("/", sleepy)],
+        middleware=[Middleware(MyMiddleware, version=_ + 1) for _ in range(10)],
+    )
+
+    scope = {
+        "type": "http",
+        "version": "3",
+        "method": "GET",
+        "path": "/",
+    }
+
+    async def receive() -> AsyncIterator[Message]:
+        yield {"type": "http.disconnect"}
+
+    sent: list[Message] = []
+
+    async def send(message: Message) -> None:
+        sent.append(message)
+
+    await app(scope, receive().__anext__, send)
+
+    assert ordered_events == [
+        "1:STARTED",
+        "2:STARTED",
+        "3:STARTED",
+        "4:STARTED",
+        "5:STARTED",
+        "6:STARTED",
+        "7:STARTED",
+        "8:STARTED",
+        "9:STARTED",
+        "10:STARTED",
+        "10:COMPLETED",
+        "9:COMPLETED",
+        "8:COMPLETED",
+        "7:COMPLETED",
+        "6:COMPLETED",
+        "5:COMPLETED",
+        "4:COMPLETED",
+        "3:COMPLETED",
+        "2:COMPLETED",
+        "1:COMPLETED",
+    ]
+
+    assert sorted(unordered_events) == sorted(
+        [
+            "1:BACKGROUND",
+            "2:BACKGROUND",
+            "3:BACKGROUND",
+            "4:BACKGROUND",
+            "5:BACKGROUND",
+            "6:BACKGROUND",
+            "7:BACKGROUND",
+            "8:BACKGROUND",
+            "9:BACKGROUND",
+            "10:BACKGROUND",
+        ]
+    )
+
+    assert sent == [
+        {
+            "type": "http.response.start",
+            "status": 200,
+            "headers": [(b"content-length", b"0")],
+        },
+        {"type": "http.response.body", "body": b"", "more_body": False},
+    ]
+
+
+@pytest.mark.anyio
+@pytest.mark.parametrize("send_body", [True, False])
+async def test_poll_for_disconnect_repeated(send_body: bool) -> None:
+    async def app_poll_disconnect(scope: Scope, receive: Receive, send: Send) -> None:
+        for _ in range(2):
+            msg = await receive()
+            while msg["type"] == "http.request":
+                msg = await receive()
+            assert msg["type"] == "http.disconnect"
+        await Response(b"good!")(scope, receive, send)
+
+    class MyMiddleware(BaseHTTPMiddleware):
+        async def dispatch(self, request: Request, call_next: RequestResponseEndpoint) -> Response:
+            return await call_next(request)
+
+    app = MyMiddleware(app_poll_disconnect)
+
+    scope = {
+        "type": "http",
+        "version": "3",
+        "method": "GET",
+        "path": "/",
+    }
+
+    async def receive() -> AsyncIterator[Message]:
+        # the key here is that we only ever send 1 htt.disconnect message
+        if send_body:
+            yield {"type": "http.request", "body": b"hello", "more_body": True}
+            yield {"type": "http.request", "body": b"", "more_body": False}
+        yield {"type": "http.disconnect"}
+        raise AssertionError("Should not be called, would hang")  # pragma: no cover
+
+    sent: list[Message] = []
+
+    async def send(message: Message) -> None:
+        sent.append(message)
+
+    await app(scope, receive().__anext__, send)
+
+    assert sent == [
+        {
+            "type": "http.response.start",
+            "status": 200,
+            "headers": [(b"content-length", b"5")],
+        },
+        {"type": "http.response.body", "body": b"good!", "more_body": True},
+        {"type": "http.response.body", "body": b"", "more_body": False},
+    ]
